@@ -5,7 +5,8 @@ use crate::{
         event_pump, CommandBlocking, CommandInfo, CommitComponent,
         Component, CreateBranchComponent, DrawableComponent,
         ExternalEditorComponent, HelpComponent,
-        InspectCommitComponent, MsgComponent, ResetComponent,
+        InspectCommitComponent, MsgComponent, PushComponent,
+        RenameBranchComponent, ResetComponent, SelectBranchComponent,
         StashMsgComponent, TagCommitComponent,
     },
     input::{Input, InputEvent, InputState},
@@ -15,7 +16,7 @@ use crate::{
     tabs::{Revlog, StashList, Stashing, Status},
     ui::style::{SharedTheme, Theme},
 };
-use anyhow::{anyhow, Result};
+use anyhow::{bail, Result};
 use asyncgit::{sync, AsyncNotification, CWD};
 use crossbeam_channel::Sender;
 use crossterm::event::{Event, KeyEvent};
@@ -27,6 +28,7 @@ use std::{
 use tui::{
     backend::Backend,
     layout::{Constraint, Direction, Layout, Margin, Rect},
+    text::{Span, Spans},
     widgets::{Block, Borders, Tabs},
     Frame,
 };
@@ -41,8 +43,11 @@ pub struct App {
     stashmsg_popup: StashMsgComponent,
     inspect_commit_popup: InspectCommitComponent,
     external_editor_popup: ExternalEditorComponent,
+    push_popup: PushComponent,
     tag_commit_popup: TagCommitComponent,
     create_branch_popup: CreateBranchComponent,
+    rename_branch_popup: RenameBranchComponent,
+    select_branch_popup: SelectBranchComponent,
     cmdbar: RefCell<CommandBar>,
     tab: usize,
     revlog: Revlog,
@@ -98,6 +103,12 @@ impl App {
                 theme.clone(),
                 key_config.clone(),
             ),
+            push_popup: PushComponent::new(
+                &queue,
+                sender,
+                theme.clone(),
+                key_config.clone(),
+            ),
             tag_commit_popup: TagCommitComponent::new(
                 queue.clone(),
                 theme.clone(),
@@ -108,6 +119,16 @@ impl App {
                 theme.clone(),
                 key_config.clone(),
             ),
+            rename_branch_popup: RenameBranchComponent::new(
+                queue.clone(),
+                theme.clone(),
+                key_config.clone(),
+            ),
+            select_branch_popup: SelectBranchComponent::new(
+                queue.clone(),
+                theme.clone(),
+                key_config.clone(),
+            ),
             do_quit: false,
             cmdbar: RefCell::new(CommandBar::new(
                 theme.clone(),
@@ -178,7 +199,7 @@ impl App {
             1 => self.revlog.draw(f, chunks_main[1])?,
             2 => self.stashing_tab.draw(f, chunks_main[1])?,
             3 => self.stashlist_tab.draw(f, chunks_main[1])?,
-            _ => return Err(anyhow!("unknown tab")),
+            _ => bail!("unknown tab"),
         };
 
         self.draw_popups(f)?;
@@ -203,9 +224,7 @@ impl App {
                 let new_flags = if k == self.key_config.tab_toggle {
                     self.toggle_tabs(false)?;
                     NeedsUpdate::COMMANDS
-                } else if k == self.key_config.tab_toggle_reverse
-                    || k == self.key_config.tab_toggle_reverse_windows
-                {
+                } else if k == self.key_config.tab_toggle_reverse {
                     self.toggle_tabs(true)?;
                     NeedsUpdate::COMMANDS
                 } else if k == self.key_config.tab_status
@@ -225,21 +244,7 @@ impl App {
                 flags.insert(new_flags);
             }
 
-            let new_flags = self.process_queue()?;
-            flags.insert(new_flags);
-
-            if flags.contains(NeedsUpdate::ALL) {
-                self.update()?;
-            }
-            //TODO: make this a queue event?
-            //NOTE: set when any tree component changed selection
-            if flags.contains(NeedsUpdate::DIFF) {
-                self.status_tab.update_diff()?;
-                self.inspect_commit_popup.update_diff()?;
-            }
-            if flags.contains(NeedsUpdate::COMMANDS) {
-                self.update_commands();
-            }
+            self.process_queue(flags)?;
         } else if let InputEvent::State(polling_state) = ev {
             self.external_editor_popup.hide();
             if let InputState::Paused = polling_state {
@@ -256,7 +261,7 @@ impl App {
                     let msg =
                         format!("failed to launch editor:\n{}", e);
                     log::error!("{}", msg.as_str());
-                    self.msg.show_msg(msg.as_str())?;
+                    self.msg.show_error(msg.as_str())?;
                 }
 
                 self.requires_redraw.set(true);
@@ -293,10 +298,11 @@ impl App {
         self.stashing_tab.update_git(ev)?;
         self.revlog.update_git(ev)?;
         self.inspect_commit_popup.update_git(ev)?;
+        self.push_popup.update_git(ev)?;
 
         //TODO: better system for this
         // can we simply process the queue here and everyone just uses the queue to schedule a cmd update?
-        self.update_commands();
+        self.process_queue(NeedsUpdate::COMMANDS)?;
 
         Ok(())
     }
@@ -337,8 +343,11 @@ impl App {
             stashmsg_popup,
             inspect_commit_popup,
             external_editor_popup,
+            push_popup,
             tag_commit_popup,
             create_branch_popup,
+            rename_branch_popup,
+            select_branch_popup,
             help,
             revlog,
             status_tab,
@@ -411,7 +420,28 @@ impl App {
         self.cmdbar.borrow_mut().set_cmds(self.commands(false));
     }
 
-    fn process_queue(&mut self) -> Result<NeedsUpdate> {
+    fn process_queue(&mut self, flags: NeedsUpdate) -> Result<()> {
+        let mut flags = flags;
+        let new_flags = self.process_internal_events()?;
+        flags.insert(new_flags);
+
+        if flags.contains(NeedsUpdate::ALL) {
+            self.update()?;
+        }
+        //TODO: make this a queue event?
+        //NOTE: set when any tree component changed selection
+        if flags.contains(NeedsUpdate::DIFF) {
+            self.status_tab.update_diff()?;
+            self.inspect_commit_popup.update_diff()?;
+        }
+        if flags.contains(NeedsUpdate::COMMANDS) {
+            self.update_commands();
+        }
+
+        Ok(())
+    }
+
+    fn process_internal_events(&mut self) -> Result<NeedsUpdate> {
         let mut flags = NeedsUpdate::empty();
 
         loop {
@@ -448,13 +478,27 @@ impl App {
                     sync::reset_hunk(CWD, path, hash)?;
                     flags.insert(NeedsUpdate::ALL);
                 }
+                Action::DeleteBranch(branch_ref) => {
+                    if let Err(e) =
+                        sync::delete_branch(CWD, &branch_ref)
+                    {
+                        self.queue.borrow_mut().push_back(
+                            InternalEvent::ShowErrorMsg(
+                                e.to_string(),
+                            ),
+                        )
+                    } else {
+                        flags.insert(NeedsUpdate::ALL);
+                        self.select_branch_popup.hide();
+                    }
+                }
             },
             InternalEvent::ConfirmAction(action) => {
                 self.reset.open(action)?;
                 flags.insert(NeedsUpdate::COMMANDS);
             }
             InternalEvent::ShowErrorMsg(msg) => {
-                self.msg.show_msg(msg.as_str())?;
+                self.msg.show_error(msg.as_str())?;
                 flags
                     .insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
             }
@@ -470,6 +514,13 @@ impl App {
             InternalEvent::CreateBranch => {
                 self.create_branch_popup.open()?;
             }
+            InternalEvent::RenameBranch(branch_ref, cur_name) => {
+                self.rename_branch_popup
+                    .open(branch_ref, cur_name)?;
+            }
+            InternalEvent::SelectBranch => {
+                self.select_branch_popup.open()?;
+            }
             InternalEvent::TabSwitch => self.set_tab(0)?,
             InternalEvent::InspectCommit(id, tags) => {
                 self.inspect_commit_popup.open(id, tags)?;
@@ -481,6 +532,10 @@ impl App {
                 self.file_to_open = path;
                 flags.insert(NeedsUpdate::COMMANDS)
             }
+            InternalEvent::Push(branch) => {
+                self.push_popup.push(branch)?;
+                flags.insert(NeedsUpdate::ALL)
+            }
         };
 
         Ok(flags)
@@ -529,6 +584,7 @@ impl App {
         res
     }
 
+    //TODO: make this automatic, i keep forgetting to add popups here
     fn any_popup_visible(&self) -> bool {
         self.commit.is_visible()
             || self.help.is_visible()
@@ -539,6 +595,9 @@ impl App {
             || self.external_editor_popup.is_visible()
             || self.tag_commit_popup.is_visible()
             || self.create_branch_popup.is_visible()
+            || self.push_popup.is_visible()
+            || self.select_branch_popup.is_visible()
+            || self.rename_branch_popup.is_visible()
     }
 
     fn draw_popups<B: Backend>(
@@ -558,13 +617,16 @@ impl App {
 
         self.commit.draw(f, size)?;
         self.stashmsg_popup.draw(f, size)?;
-        self.reset.draw(f, size)?;
         self.help.draw(f, size)?;
         self.inspect_commit_popup.draw(f, size)?;
-        self.msg.draw(f, size)?;
         self.external_editor_popup.draw(f, size)?;
         self.tag_commit_popup.draw(f, size)?;
+        self.select_branch_popup.draw(f, size)?;
         self.create_branch_popup.draw(f, size)?;
+        self.rename_branch_popup.draw(f, size)?;
+        self.push_popup.draw(f, size)?;
+        self.reset.draw(f, size)?;
+        self.msg.draw(f, size)?;
 
         Ok(())
     }
@@ -576,24 +638,27 @@ impl App {
             horizontal: 1,
         });
 
-        let tabs = &[
-            strings::tab_status(&self.key_config),
-            strings::tab_log(&self.key_config),
-            strings::tab_stashing(&self.key_config),
-            strings::tab_stashes(&self.key_config),
-        ];
+        let tabs = [
+            Span::raw(strings::tab_status(&self.key_config)),
+            Span::raw(strings::tab_log(&self.key_config)),
+            Span::raw(strings::tab_stashing(&self.key_config)),
+            Span::raw(strings::tab_stashes(&self.key_config)),
+        ]
+        .iter()
+        .cloned()
+        .map(Spans::from)
+        .collect();
 
         f.render_widget(
-            Tabs::default()
+            Tabs::new(tabs)
                 .block(
                     Block::default()
                         .borders(Borders::BOTTOM)
                         .border_style(self.theme.block(false)),
                 )
-                .titles(tabs)
                 .style(self.theme.tab(false))
                 .highlight_style(self.theme.tab(true))
-                .divider(&strings::tab_divider(&self.key_config))
+                .divider(strings::tab_divider(&self.key_config))
                 .select(self.tab),
             r,
         );
