@@ -11,18 +11,16 @@ use crate::{
     ui::style::SharedTheme,
 };
 use anyhow::Result;
-use asyncgit::{cached, sync, StatusItem, StatusItemType, CWD};
+use asyncgit::{sync, StatusItem, StatusItemType, CWD};
 use crossterm::event::Event;
 use std::path::Path;
 use tui::{backend::Backend, layout::Rect, Frame};
 
 ///
 pub struct ChangesComponent {
-    title: String,
     files: FileTreeComponent,
     is_working_dir: bool,
     queue: Queue,
-    branch_name: cached::BranchName,
     key_config: SharedKeyConfig,
 }
 
@@ -37,7 +35,6 @@ impl ChangesComponent {
         key_config: SharedKeyConfig,
     ) -> Self {
         Self {
-            title: title.into(),
             files: FileTreeComponent::new(
                 title,
                 focus,
@@ -47,23 +44,10 @@ impl ChangesComponent {
             ),
             is_working_dir,
             queue,
-            branch_name: cached::BranchName::new(CWD),
             key_config,
         }
     }
 
-    pub fn update(&mut self) -> Result<()> {
-        if self.is_working_dir {
-            if let Ok(branch_name) = self.branch_name.lookup() {
-                self.files.set_title(format!(
-                    "{} - {{{}}}",
-                    &self.title, branch_name,
-                ))
-            }
-        }
-        Ok(())
-    }
-
     ///
     pub fn set_items(&mut self, list: &[StatusItem]) -> Result<()> {
         self.files.update(list)?;
