@@ -17,7 +17,7 @@ use anyhow::Result;
 use asyncgit::{hash, StatusItem, StatusItemType};
 use crossterm::event::Event;
 use std::{borrow::Cow, cell::Cell, convert::From, path::Path};
-use tui::{backend::Backend, layout::Rect, widgets::Text, Frame};
+use tui::{backend::Backend, layout::Rect, text::Span, Frame};
 
 ///
 pub struct FileTreeComponent {
@@ -135,23 +135,36 @@ impl FileTreeComponent {
         changed
     }
 
+    const fn item_status_char(item_type: StatusItemType) -> char {
+        match item_type {
+            StatusItemType::Modified => 'M',
+            StatusItemType::New => '+',
+            StatusItemType::Deleted => '-',
+            StatusItemType::Renamed => 'R',
+            StatusItemType::Typechange => ' ',
+        }
+    }
+
     fn item_to_text<'b>(
-        item: &FileTreeItem,
+        string: &str,
+        indent: usize,
+        visible: bool,
+        file_item_kind: &FileTreeItemKind,
         width: u16,
         selected: bool,
         theme: &'b SharedTheme,
-    ) -> Option<Text<'b>> {
-        let indent_str = if item.info.indent == 0 {
+    ) -> Option<Span<'b>> {
+        let indent_str = if indent == 0 {
             String::from("")
         } else {
-            format!("{:w$}", " ", w = (item.info.indent as usize) * 2)
+            format!("{:w$}", " ", w = (indent as usize) * 2)
         };
 
-        if !item.info.visible {
+        if !visible {
             return None;
         }
 
-        match &item.kind {
+        match file_item_kind {
             FileTreeItemKind::File(status_item) => {
                 let status_char =
                     Self::item_status_char(status_item.status);
@@ -172,7 +185,7 @@ impl FileTreeComponent {
                     format!("{} {}{}", status_char, indent_str, file)
                 };
 
-                Some(Text::Styled(
+                Some(Span::styled(
                     Cow::from(txt),
                     theme.item(status_item.status, selected),
                 ))
@@ -187,17 +200,17 @@ impl FileTreeComponent {
                         "  {}{}{:w$}",
                         indent_str,
                         collapse_char,
-                        item.info.path,
+                        string,
                         w = width as usize
                     )
                 } else {
                     format!(
                         "  {}{}{}",
-                        indent_str, collapse_char, item.info.path,
+                        indent_str, collapse_char, string,
                     )
                 };
 
-                Some(Text::Styled(
+                Some(Span::styled(
                     Cow::from(txt),
                     theme.text(true, selected),
                 ))
@@ -205,17 +218,93 @@ impl FileTreeComponent {
         }
     }
 
-    const fn item_status_char(item_type: StatusItemType) -> char {
-        match item_type {
-            StatusItemType::Modified => 'M',
-            StatusItemType::New => '+',
-            StatusItemType::Deleted => '-',
-            StatusItemType::Renamed => 'R',
-            StatusItemType::Typechange => ' ',
+    /// Returns a Vec<TextDrawInfo> which is used to draw the `FileTreeComponent` correctly,
+    /// allowing folders to be folded up if they are alone in their directory
+    fn build_vec_text_draw_info_for_drawing(
+        &self,
+    ) -> (Vec<TextDrawInfo>, usize, usize) {
+        let mut should_skip_over: usize = 0;
+        let mut selection_offset: usize = 0;
+        let mut selection_offset_visible: usize = 0;
+        let mut vec_draw_text_info: Vec<TextDrawInfo> = vec![];
+        let tree_items = self.tree.tree.items();
+
+        for (index, item) in tree_items.iter().enumerate() {
+            if should_skip_over > 0 {
+                should_skip_over -= 1;
+                continue;
+            }
+
+            let index_above_select =
+                index < self.tree.selection.unwrap_or(0);
+
+            if !item.info.visible && index_above_select {
+                selection_offset_visible += 1;
+            }
+
+            vec_draw_text_info.push(TextDrawInfo {
+                name: item.info.path.clone(),
+                indent: item.info.indent,
+                visible: item.info.visible,
+                item_kind: &item.kind,
+            });
+
+            let mut idx_temp = index;
+
+            while idx_temp < tree_items.len().saturating_sub(2)
+                && tree_items[idx_temp].info.indent
+                    < tree_items[idx_temp + 1].info.indent
+            {
+                // fold up the folder/file
+                idx_temp += 1;
+                should_skip_over += 1;
+
+                // don't fold files up
+                if let FileTreeItemKind::File(_) =
+                    &tree_items[idx_temp].kind
+                {
+                    should_skip_over -= 1;
+                    break;
+                }
+                // don't fold up if more than one folder in folder
+                else if self
+                    .tree
+                    .tree
+                    .multiple_items_at_path(idx_temp)
+                {
+                    should_skip_over -= 1;
+                    break;
+                } else {
+                    // There is only one item at this level (i.e only one folder in the folder),
+                    // so do fold up
+
+                    let vec_draw_text_info_len =
+                        vec_draw_text_info.len();
+                    vec_draw_text_info[vec_draw_text_info_len - 1]
+                        .name += &(String::from("/")
+                        + &tree_items[idx_temp].info.path);
+                    if index_above_select {
+                        selection_offset += 1;
+                    }
+                }
+            }
         }
+        (
+            vec_draw_text_info,
+            selection_offset,
+            selection_offset_visible,
+        )
     }
 }
 
+/// Used for drawing the `FileTreeComponent`
+struct TextDrawInfo<'a> {
+    name: String,
+    indent: u8,
+    visible: bool,
+    item_kind: &'a FileTreeItemKind,
+}
+
 impl DrawableComponent for FileTreeComponent {
     fn draw<B: Backend>(
         &self,
@@ -223,7 +312,7 @@ impl DrawableComponent for FileTreeComponent {
         r: Rect,
     ) -> Result<()> {
         if self.pending {
-            let items = vec![Text::Styled(
+            let items = vec![Span::styled(
                 Cow::from(strings::loading_text(&self.key_config)),
                 self.theme.text(false, false),
             )];
@@ -238,21 +327,11 @@ impl DrawableComponent for FileTreeComponent {
                 &self.theme,
             );
         } else {
-            let selection_offset =
-                self.tree.tree.items().iter().enumerate().fold(
-                    0,
-                    |acc, (idx, e)| {
-                        let visible = e.info.visible;
-                        let index_above_select =
-                            idx < self.tree.selection.unwrap_or(0);
-
-                        if !visible && index_above_select {
-                            acc + 1
-                        } else {
-                            acc
-                        }
-                    },
-                );
+            let (
+                vec_draw_text_info,
+                selection_offset,
+                selection_offset_visible,
+            ) = self.build_vec_text_draw_info_for_drawing();
 
             let select = self
                 .tree
@@ -264,29 +343,24 @@ impl DrawableComponent for FileTreeComponent {
             self.scroll_top.set(ui::calc_scroll_top(
                 self.scroll_top.get(),
                 tree_height,
-                select,
+                select.saturating_sub(selection_offset_visible),
             ));
 
-            let items = self
-                .tree
-                .tree
-                .items()
+            let items = vec_draw_text_info
                 .iter()
                 .enumerate()
-                .filter_map(|(idx, e)| {
+                .filter_map(|(index, draw_text_info)| {
                     Self::item_to_text(
-                        e,
+                        &draw_text_info.name,
+                        draw_text_info.indent as usize,
+                        draw_text_info.visible,
+                        draw_text_info.item_kind,
                         r.width,
-                        self.show_selection
-                            && self
-                                .tree
-                                .selection
-                                .map_or(false, |e| e == idx),
+                        self.show_selection && select == index,
                         &self.theme,
                     )
                 })
                 .skip(self.scroll_top.get());
-
             ui::draw_list(
                 f,
                 r,
@@ -356,3 +430,101 @@ impl Component for FileTreeComponent {
         self.show_selection(focus);
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use asyncgit::StatusItemType;
+
+    fn string_vec_to_status(items: &[&str]) -> Vec<StatusItem> {
+        items
+            .iter()
+            .map(|a| StatusItem {
+                path: String::from(*a),
+                status: StatusItemType::Modified,
+            })
+            .collect::<Vec<_>>()
+    }
+
+    #[test]
+    fn test_correct_scroll_position() {
+        let items = string_vec_to_status(&[
+            "a/b/b1", //
+            "a/b/b2", //
+            "a/c/c1", //
+        ]);
+
+        //0 a/
+        //1   b/
+        //2     b1
+        //3     b2
+        //4  c/
+        //5    c1
+
+        // Set up test terminal
+        let test_backend = tui::backend::TestBackend::new(100, 100);
+        let mut terminal = tui::Terminal::new(test_backend)
+            .expect("Unable to set up terminal");
+        let mut frame = terminal.get_frame();
+
+        // set up file tree
+        let mut ftc = FileTreeComponent::new(
+            "title",
+            true,
+            None,
+            SharedTheme::default(),
+            SharedKeyConfig::default(),
+        );
+        ftc.update(&items)
+            .expect("Updating FileTreeComponent failed");
+
+        ftc.move_selection(MoveSelection::Down); // Move to b/
+        ftc.move_selection(MoveSelection::Left); // Fold b/
+        ftc.move_selection(MoveSelection::Down); // Move to c/
+
+        ftc.draw(&mut frame, Rect::new(0, 0, 10, 5))
+            .expect("Draw failed");
+
+        assert_eq!(ftc.scroll_top.get(), 0); // should still be at top
+    }
+
+    #[test]
+    fn test_correct_foldup_and_not_visible_scroll_position() {
+        let items = string_vec_to_status(&[
+            "a/b/b1", //
+            "c/d1",   //
+            "c/d2",   //
+        ]);
+
+        //0 a/b/
+        //2     b1
+        //3 c/
+        //4   d1
+        //5   d2
+
+        // Set up test terminal
+        let test_backend = tui::backend::TestBackend::new(100, 100);
+        let mut terminal = tui::Terminal::new(test_backend)
+            .expect("Unable to set up terminal");
+        let mut frame = terminal.get_frame();
+
+        // set up file tree
+        let mut ftc = FileTreeComponent::new(
+            "title",
+            true,
+            None,
+            SharedTheme::default(),
+            SharedKeyConfig::default(),
+        );
+        ftc.update(&items)
+            .expect("Updating FileTreeComponent failed");
+
+        ftc.move_selection(MoveSelection::Left); // Fold a/b/
+        ftc.move_selection(MoveSelection::Down); // Move to c/
+
+        ftc.draw(&mut frame, Rect::new(0, 0, 10, 5))
+            .expect("Draw failed");
+
+        assert_eq!(ftc.scroll_top.get(), 0); // should still be at top
+    }
+}
