@@ -4,21 +4,22 @@ mod commit;
 mod commit_details;
 mod commitlist;
 mod create_branch;
+mod cred;
 mod diff;
 mod externaleditor;
 mod filetree;
 mod help;
 mod inspect_commit;
 mod msg;
+mod push;
+mod rename_branch;
 mod reset;
+mod select_branch;
 mod stashmsg;
 mod tag_commit;
 mod textinput;
 mod utils;
 
-use anyhow::Result;
-use crossterm::event::Event;
-
 pub use changes::ChangesComponent;
 pub use command::{CommandInfo, CommandText};
 pub use commit::CommitComponent;
@@ -31,17 +32,23 @@ pub use filetree::FileTreeComponent;
 pub use help::HelpComponent;
 pub use inspect_commit::InspectCommitComponent;
 pub use msg::MsgComponent;
+pub use push::PushComponent;
+pub use rename_branch::RenameBranchComponent;
 pub use reset::ResetComponent;
+pub use select_branch::SelectBranchComponent;
 pub use stashmsg::StashMsgComponent;
 pub use tag_commit::TagCommitComponent;
-pub use textinput::TextInputComponent;
+pub use textinput::{InputType, TextInputComponent};
 pub use utils::filetree::FileTreeItemKind;
 
 use crate::ui::style::Theme;
+use anyhow::Result;
+use crossterm::event::Event;
 use tui::{
     backend::Backend,
     layout::{Alignment, Rect},
-    widgets::{Block, BorderType, Borders, Paragraph, Text},
+    text::{Span, Text},
+    widgets::{Block, BorderType, Borders, Paragraph, Wrap},
     Frame,
 };
 
@@ -181,44 +188,39 @@ pub trait Component {
     }
 }
 
-fn dialog_paragraph<'a, 't, T>(
+fn dialog_paragraph<'a>(
     title: &'a str,
-    content: T,
+    content: Text<'a>,
     theme: &Theme,
     focused: bool,
-) -> Paragraph<'a, 't, T>
-where
-    T: Iterator<Item = &'t Text<'t>>,
-{
+) -> Paragraph<'a> {
     Paragraph::new(content)
         .block(
             Block::default()
-                .title(title)
+                .title(Span::styled(title, theme.title(focused)))
                 .borders(Borders::ALL)
-                .title_style(theme.title(focused))
                 .border_style(theme.block(focused)),
         )
         .alignment(Alignment::Left)
 }
 
-fn popup_paragraph<'a, 't, T>(
+fn popup_paragraph<'a, T>(
     title: &'a str,
     content: T,
     theme: &Theme,
     focused: bool,
-) -> Paragraph<'a, 't, T>
+) -> Paragraph<'a>
 where
-    T: Iterator<Item = &'t Text<'t>>,
+    T: Into<Text<'a>>,
 {
-    Paragraph::new(content)
+    Paragraph::new(content.into())
         .block(
             Block::default()
-                .title(title)
+                .title(Span::styled(title, theme.title(focused)))
                 .borders(Borders::ALL)
                 .border_type(BorderType::Thick)
-                .title_style(theme.title(focused))
                 .border_style(theme.block(focused)),
         )
         .alignment(Alignment::Left)
-        .wrap(true)
+        .wrap(Wrap { trim: true })
 }
