@@ -1,3 +1,4 @@
+use crate::ui::Size;
 use crate::{
     components::{
         popup_paragraph, visibility_blocking, CommandBlocking,
@@ -9,14 +10,24 @@ use crate::{
 };
 use anyhow::Result;
 use crossterm::event::{Event, KeyCode, KeyModifiers};
+use itertools::Itertools;
+use std::{collections::HashMap, ops::Range};
 use tui::{
     backend::Backend,
     layout::Rect,
     style::Modifier,
-    widgets::{Clear, Text},
+    text::{Spans, Text},
+    widgets::Clear,
     Frame,
 };
 
+#[derive(PartialEq)]
+pub enum InputType {
+    Singleline,
+    Multiline,
+    Password,
+}
+
 /// primarily a subcomponet for user input of text (used in `CommitComponent`)
 pub struct TextInputComponent {
     title: String,
@@ -26,6 +37,7 @@ pub struct TextInputComponent {
     theme: SharedTheme,
     key_config: SharedKeyConfig,
     cursor_position: usize,
+    input_type: InputType,
 }
 
 impl TextInputComponent {
@@ -37,16 +49,25 @@ impl TextInputComponent {
         default_msg: &str,
     ) -> Self {
         Self {
-            msg: String::default(),
+            msg: String::new(),
             visible: false,
             theme,
             key_config,
             title: title.to_string(),
             default_msg: default_msg.to_string(),
             cursor_position: 0,
+            input_type: InputType::Multiline,
         }
     }
 
+    pub const fn with_input_type(
+        mut self,
+        input_type: InputType,
+    ) -> Self {
+        self.input_type = input_type;
+        self
+    }
+
     /// Clear the `msg`.
     pub fn clear(&mut self) {
         self.msg.clear();
@@ -108,50 +129,106 @@ impl TextInputComponent {
         self.title = t;
     }
 
-    fn get_draw_text(&self) -> Vec<Text> {
+    fn get_draw_text(&self) -> Text {
         let style = self.theme.text(true, false);
 
-        let mut txt = Vec::new();
-
-        // the portion of the text before the cursor is added
-        // if the cursor is not at the first character
+        let mut txt = Text::default();
+        // The portion of the text before the cursor is added
+        // if the cursor is not at the first character.
         if self.cursor_position > 0 {
-            txt.push(Text::styled(
-                &self.msg[..self.cursor_position],
-                style,
-            ));
+            let text_before_cursor =
+                self.get_msg(0..self.cursor_position);
+            let ends_in_nl = text_before_cursor.ends_with('\n');
+            txt = text_append(
+                txt,
+                Text::styled(text_before_cursor, style),
+            );
+            if ends_in_nl {
+                txt.lines.push(Spans::default());
+                // txt = text_append(txt, Text::styled("\n\r", style));
+            }
         }
 
         let cursor_str = self
             .next_char_position()
             // if the cursor is at the end of the msg
             // a whitespace is used to underline
-            .map_or(" ", |pos| &self.msg[self.cursor_position..pos]);
+            .map_or(" ".to_owned(), |pos| {
+                self.get_msg(self.cursor_position..pos)
+            });
 
-        if cursor_str == "\n" {
-            txt.push(Text::styled(
-                "\u{21b5}",
-                self.theme
-                    .text(false, false)
-                    .modifier(Modifier::UNDERLINED),
-            ));
+        let cursor_highlighting = {
+            let mut h = HashMap::with_capacity(2);
+            h.insert("\n", "\u{21b5}\n\r");
+            h.insert(" ", "\u{00B7}");
+            h
+        };
+
+        if let Some(substitute) =
+            cursor_highlighting.get(cursor_str.as_str())
+        {
+            txt = text_append(
+                txt,
+                Text::styled(
+                    substitute.to_owned(),
+                    self.theme
+                        .text(false, false)
+                        .add_modifier(Modifier::UNDERLINED),
+                ),
+            );
+        } else {
+            txt = text_append(
+                txt,
+                Text::styled(
+                    cursor_str,
+                    style.add_modifier(Modifier::UNDERLINED),
+                ),
+            );
         }
 
-        txt.push(Text::styled(
-            cursor_str,
-            style.modifier(Modifier::UNDERLINED),
-        ));
-
-        // the final portion of the text is added if there is
-        // still remaining characters
+        // The final portion of the text is added if there are
+        // still remaining characters.
         if let Some(pos) = self.next_char_position() {
             if pos < self.msg.len() {
-                txt.push(Text::styled(&self.msg[pos..], style));
+                txt = text_append(
+                    txt,
+                    Text::styled(
+                        self.get_msg(pos..self.msg.len()),
+                        style,
+                    ),
+                );
             }
         }
 
         txt
     }
+
+    fn get_msg(&self, range: Range<usize>) -> String {
+        match self.input_type {
+            InputType::Password => range.map(|_| "*").join(""),
+            _ => self.msg[range].to_owned(),
+        }
+    }
+}
+
+// merges last line of `txt` with first of `append` so we do not generate unneeded newlines
+fn text_append<'a>(txt: Text<'a>, append: Text<'a>) -> Text<'a> {
+    let mut txt = txt;
+    if let Some(last_line) = txt.lines.last_mut() {
+        if let Some(first_line) = append.lines.first() {
+            last_line.0.extend(first_line.0.clone());
+        }
+
+        if append.lines.len() > 1 {
+            for line in 1..append.lines.len() {
+                let spans = append.lines[line].clone();
+                txt.lines.push(spans);
+            }
+        }
+    } else {
+        txt = append
+    }
+    txt
 }
 
 impl DrawableComponent for TextInputComponent {
@@ -162,22 +239,31 @@ impl DrawableComponent for TextInputComponent {
     ) -> Result<()> {
         if self.visible {
             let txt = if self.msg.is_empty() {
-                vec![Text::styled(
+                Text::styled(
                     self.default_msg.as_str(),
                     self.theme.text(false, false),
-                )]
+                )
             } else {
                 self.get_draw_text()
             };
 
-            let area = ui::centered_rect(60, 20, f.size());
-            let area = ui::rect_min(10, 3, area);
+            let area = match self.input_type {
+                InputType::Multiline => {
+                    let area = ui::centered_rect(60, 20, f.size());
+                    ui::rect_inside(
+                        Size::new(10, 3),
+                        f.size().into(),
+                        area,
+                    )
+                }
+                _ => ui::centered_rect_absolute(32, 3, f.size()),
+            };
 
             f.render_widget(Clear, area);
             f.render_widget(
                 popup_paragraph(
                     self.title.as_str(),
-                    txt.iter(),
+                    txt,
                     &self.theme,
                     true,
                 ),
@@ -274,6 +360,7 @@ impl Component for TextInputComponent {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use tui::{style::Style, text::Span};
 
     #[test]
     fn test_smoke() {
@@ -295,12 +382,59 @@ mod tests {
         assert_eq!(comp.cursor_position, 0);
     }
 
-    fn get_text<'a>(t: &'a Text) -> Option<&'a str> {
-        if let Text::Styled(c, _) = t {
-            Some(c.as_ref())
-        } else {
-            None
-        }
+    #[test]
+    fn text_cursor_initial_position() {
+        let mut comp = TextInputComponent::new(
+            SharedTheme::default(),
+            SharedKeyConfig::default(),
+            "",
+            "",
+        );
+        let theme = SharedTheme::default();
+        let underlined = theme
+            .text(true, false)
+            .add_modifier(Modifier::UNDERLINED);
+
+        comp.set_text(String::from("a"));
+
+        let txt = comp.get_draw_text();
+
+        assert_eq!(txt.lines[0].0.len(), 1);
+        assert_eq!(get_text(&txt.lines[0].0[0]), Some("a"));
+        assert_eq!(get_style(&txt.lines[0].0[0]), Some(&underlined));
+    }
+
+    #[test]
+    fn test_cursor_second_position() {
+        let mut comp = TextInputComponent::new(
+            SharedTheme::default(),
+            SharedKeyConfig::default(),
+            "",
+            "",
+        );
+        let theme = SharedTheme::default();
+        let underlined_whitespace = theme
+            .text(false, false)
+            .add_modifier(Modifier::UNDERLINED);
+
+        let not_underlined = Style::default();
+
+        comp.set_text(String::from("a"));
+        comp.incr_cursor();
+
+        let txt = comp.get_draw_text();
+
+        assert_eq!(txt.lines[0].0.len(), 2);
+        assert_eq!(get_text(&txt.lines[0].0[0]), Some("a"));
+        assert_eq!(
+            get_style(&txt.lines[0].0[0]),
+            Some(&not_underlined)
+        );
+        assert_eq!(get_text(&txt.lines[0].0[1]), Some("\u{00B7}"));
+        assert_eq!(
+            get_style(&txt.lines[0].0[1]),
+            Some(&underlined_whitespace)
+        );
     }
 
     #[test]
@@ -312,16 +446,58 @@ mod tests {
             "",
         );
 
-        comp.set_text(String::from("a\nb"));
+        let theme = SharedTheme::default();
+        let underlined = theme
+            .text(false, false)
+            .add_modifier(Modifier::UNDERLINED);
 
+        comp.set_text(String::from("a\nb"));
         comp.incr_cursor();
 
         let txt = comp.get_draw_text();
 
-        assert_eq!(txt.len(), 4);
-        assert_eq!(get_text(&txt[0]), Some("a"));
-        assert_eq!(get_text(&txt[1]), Some("\u{21b5}"));
-        assert_eq!(get_text(&txt[2]), Some("\n"));
-        assert_eq!(get_text(&txt[3]), Some("b"));
+        assert_eq!(txt.lines.len(), 2);
+        assert_eq!(txt.lines[0].0.len(), 2);
+        assert_eq!(txt.lines[1].0.len(), 2);
+        assert_eq!(get_text(&txt.lines[0].0[0]), Some("a"));
+        assert_eq!(get_text(&txt.lines[0].0[1]), Some("\u{21b5}"));
+        assert_eq!(get_style(&txt.lines[0].0[1]), Some(&underlined));
+        assert_eq!(get_text(&txt.lines[1].0[0]), Some(""));
+        assert_eq!(get_text(&txt.lines[1].0[1]), Some("b"));
+    }
+
+    #[test]
+    fn test_invisable_newline() {
+        let mut comp = TextInputComponent::new(
+            SharedTheme::default(),
+            SharedKeyConfig::default(),
+            "",
+            "",
+        );
+
+        let theme = SharedTheme::default();
+        let underlined = theme
+            .text(true, false)
+            .add_modifier(Modifier::UNDERLINED);
+
+        comp.set_text(String::from("a\nb"));
+
+        let txt = comp.get_draw_text();
+
+        assert_eq!(txt.lines.len(), 2);
+        assert_eq!(txt.lines[0].0.len(), 2);
+        assert_eq!(txt.lines[1].0.len(), 1);
+        assert_eq!(get_text(&txt.lines[0].0[0]), Some("a"));
+        assert_eq!(get_text(&txt.lines[0].0[1]), Some(""));
+        assert_eq!(get_style(&txt.lines[0].0[0]), Some(&underlined));
+        assert_eq!(get_text(&txt.lines[1].0[0]), Some("b"));
+    }
+
+    fn get_text<'a>(t: &'a Span) -> Option<&'a str> {
+        Some(&t.content)
+    }
+
+    fn get_style<'a>(t: &'a Span) -> Option<&'a Style> {
+        Some(&t.style)
     }
 }
