@@ -1,13 +1,14 @@
 #![forbid(unsafe_code)]
+#![deny(unused_imports)]
 #![deny(clippy::cargo)]
 #![deny(clippy::pedantic)]
 #![deny(clippy::perf)]
 #![deny(clippy::nursery)]
 #![deny(clippy::unwrap_used)]
 #![deny(clippy::panic)]
+#![deny(clippy::match_like_matches_macro)]
 #![allow(clippy::module_name_repetitions)]
 #![allow(clippy::multiple_crate_versions)]
-#![warn(clippy::missing_const_for_fn)]
 
 mod app;
 mod clipboard;
@@ -25,7 +26,7 @@ mod ui;
 mod version;
 
 use crate::app::App;
-use anyhow::{anyhow, Result};
+use anyhow::{anyhow, bail, Result};
 use asyncgit::AsyncNotification;
 use backtrace::Backtrace;
 use clap::{
@@ -82,12 +83,6 @@ fn main() -> Result<()> {
         return Ok(());
     }
 
-    // TODO: Remove this when upgrading from v0.8.x is unlikely
-    // Only run this migration on macOS, as it's the only platform where the config needs to be moved
-    if cfg!(target_os = "macos") {
-        migrate_config()?;
-    }
-
     setup_terminal()?;
     defer! {
         shutdown_terminal().expect("shutdown failed");
@@ -218,7 +213,7 @@ fn select_event(
         1 => oper.recv(rx_git).map(QueueEvent::GitEvent),
         2 => oper.recv(rx_ticker).map(|_| QueueEvent::Tick),
         3 => oper.recv(rx_spinner).map(|_| QueueEvent::SpinnerUpdate),
-        _ => return Err(anyhow!("unknown select source")),
+        _ => bail!("unknown select source"),
     }?;
 
     Ok(ev)
@@ -236,7 +231,7 @@ fn start_terminal<W: Write>(
 }
 
 fn get_app_cache_path() -> Result<PathBuf> {
-    let mut path = dirs::cache_dir()
+    let mut path = dirs_next::cache_dir()
         .ok_or_else(|| anyhow!("failed to find os cache dir."))?;
 
     path.push("gitui");
@@ -245,7 +240,7 @@ fn get_app_cache_path() -> Result<PathBuf> {
 }
 
 fn get_app_config_path() -> Result<PathBuf> {
-    let mut path = dirs::config_dir()
+    let mut path = dirs_next::config_dir()
         .ok_or_else(|| anyhow!("failed to find os config dir."))?;
 
     path.push("gitui");
@@ -253,29 +248,6 @@ fn get_app_config_path() -> Result<PathBuf> {
     Ok(path)
 }
 
-fn migrate_config() -> Result<()> {
-    let mut path = dirs::preference_dir().ok_or_else(|| {
-        anyhow!("failed to find os preference dir.")
-    })?;
-
-    path.push("gitui");
-    if !path.exists() {
-        return Ok(());
-    }
-
-    let config_path = get_app_config_path()?;
-    let entries = path.read_dir()?.flatten();
-    for entry in entries {
-        let mut config_path = config_path.clone();
-        config_path.push(entry.file_name());
-        fs::rename(entry.path(), config_path)?;
-    }
-
-    let _ = fs::remove_dir(path);
-
-    Ok(())
-}
-
 fn setup_logging() -> Result<()> {
     let mut path = get_app_cache_path()?;
     path.push("gitui.log");
