@@ -28,6 +28,7 @@ pub enum Action {
     Reset(ResetItem),
     ResetHunk(String, u64),
     StashDrop(CommitId),
+    DeleteBranch(String),
 }
 
 ///
@@ -53,7 +54,13 @@ pub enum InternalEvent {
     ///
     CreateBranch,
     ///
+    RenameBranch(String, String),
+    ///
+    SelectBranch,
+    ///
     OpenExternalEditor(Option<String>),
+    ///
+    Push(String),
 }
 
 ///
