@@ -4,6 +4,14 @@ pub mod order {
     pub static NAV: i8 = 1;
 }
 
+pub static PUSH_POPUP_MSG: &str = "Push";
+pub static PUSH_POPUP_PROGRESS_NONE: &str = "preparing...";
+pub static PUSH_POPUP_STATES_ADDING: &str = "adding objects (1/3)";
+pub static PUSH_POPUP_STATES_DELTAS: &str = "deltas (2/3)";
+pub static PUSH_POPUP_STATES_PUSHING: &str = "pushing (3/3)";
+
+pub static SELECT_BRANCH_POPUP_MSG: &str = "Switch Branch";
+
 pub fn title_status(key_config: &SharedKeyConfig) -> String {
     format!(
         "Unstaged Changes [{}]",
@@ -82,6 +90,17 @@ pub fn confirm_msg_resethunk(
 ) -> String {
     "confirm reset hunk?".to_string()
 }
+pub fn confirm_title_delete_branch(
+    _key_config: &SharedKeyConfig,
+) -> String {
+    "Delete Branch".to_string()
+}
+pub fn confirm_msg_delete_branch(
+    _key_config: &SharedKeyConfig,
+    branch_ref: &str,
+) -> String {
+    format!("Confirm deleting branch: '{}' ?", branch_ref)
+}
 pub fn log_title(_key_config: &SharedKeyConfig) -> String {
     "Commit".to_string()
 }
@@ -120,6 +139,29 @@ pub fn create_branch_popup_msg(
 ) -> String {
     "type branch name".to_string()
 }
+pub fn username_popup_title(_key_config: &SharedKeyConfig) -> String {
+    "Username".to_string()
+}
+pub fn username_popup_msg(_key_config: &SharedKeyConfig) -> String {
+    "type username".to_string()
+}
+pub fn password_popup_title(_key_config: &SharedKeyConfig) -> String {
+    "Password".to_string()
+}
+pub fn password_popup_msg(_key_config: &SharedKeyConfig) -> String {
+    "type password".to_string()
+}
+
+pub fn rename_branch_popup_title(
+    _key_config: &SharedKeyConfig,
+) -> String {
+    "Rename Branch".to_string()
+}
+pub fn rename_branch_popup_msg(
+    _key_config: &SharedKeyConfig,
+) -> String {
+    "new branch name".to_string()
+}
 
 pub mod commit {
     use crate::keys::SharedKeyConfig;
@@ -244,6 +286,13 @@ pub mod commands {
             CMD_GROUP_DIFF,
         )
     }
+    pub fn copy_hash(key_config: &SharedKeyConfig) -> CommandText {
+        CommandText::new(
+            format!("Copy Hash [{}]", get_hint(key_config.copy),),
+            "copy selected commit hash to clipboard",
+            CMD_GROUP_DIFF,
+        )
+    }
     pub fn diff_home_end(
         key_config: &SharedKeyConfig,
     ) -> CommandText {
@@ -304,6 +353,14 @@ pub mod commands {
         )
         .hide_help()
     }
+    pub fn validate_msg(key_config: &SharedKeyConfig) -> CommandText {
+        CommandText::new(
+            format!("Validate [{}]", get_hint(key_config.enter),),
+            "validate msg",
+            CMD_GROUP_GENERAL,
+        )
+        .hide_help()
+    }
     pub fn select_staging(
         key_config: &SharedKeyConfig,
     ) -> CommandText {
@@ -589,11 +646,64 @@ pub mod commands {
     ) -> CommandText {
         CommandText::new(
             format!(
-                "Branch [{}]",
+                "Create [{}]",
                 get_hint(key_config.create_branch),
             ),
             "open create branch popup",
             CMD_GROUP_GENERAL,
         )
     }
+    pub fn rename_branch_confirm_msg(
+        key_config: &SharedKeyConfig,
+    ) -> CommandText {
+        CommandText::new(
+            format!("Rename Branch [{}]", get_hint(key_config.enter),),
+            "rename branch",
+            CMD_GROUP_GENERAL,
+        )
+    }
+    pub fn rename_branch_popup(
+        key_config: &SharedKeyConfig,
+    ) -> CommandText {
+        CommandText::new(
+            format!(
+                "Rename Branch [{}]",
+                get_hint(key_config.rename_branch),
+            ),
+            "rename branch",
+            CMD_GROUP_GENERAL,
+        )
+    }
+    pub fn delete_branch_popup(
+        key_config: &SharedKeyConfig,
+    ) -> CommandText {
+        CommandText::new(
+            format!(
+                "Delete [{}]",
+                get_hint(key_config.delete_branch),
+            ),
+            "delete a branch",
+            CMD_GROUP_GENERAL,
+        )
+    }
+    pub fn open_branch_select_popup(
+        key_config: &SharedKeyConfig,
+    ) -> CommandText {
+        CommandText::new(
+            format!(
+                "Branches [{}]",
+                get_hint(key_config.select_branch),
+            ),
+            "open select branch popup",
+            CMD_GROUP_GENERAL,
+        )
+    }
+
+    pub fn status_push(key_config: &SharedKeyConfig) -> CommandText {
+        CommandText::new(
+            format!("Push [{}]", get_hint(key_config.push),),
+            "push to origin",
+            CMD_GROUP_GENERAL,
+        )
+    }
 }
