@@ -12,13 +12,18 @@ use crate::{
 };
 use anyhow::Result;
 use asyncgit::{
+    cached,
+    sync::BranchCompare,
     sync::{self, status::StatusType},
     AsyncDiff, AsyncNotification, AsyncStatus, DiffParams, DiffType,
     StatusParams, CWD,
 };
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
-use tui::layout::{Constraint, Direction, Layout};
+use tui::{
+    layout::{Alignment, Constraint, Direction, Layout},
+    widgets::Paragraph,
+};
 
 ///
 #[derive(PartialEq)]
@@ -45,6 +50,8 @@ pub struct Status {
     git_diff: AsyncDiff,
     git_status_workdir: AsyncStatus,
     git_status_stage: AsyncStatus,
+    git_branch_state: Option<BranchCompare>,
+    git_branch_name: cached::BranchName,
     queue: Queue,
     git_action_executed: bool,
     key_config: SharedKeyConfig,
@@ -95,6 +102,7 @@ impl DrawableComponent for Status {
         self.index_wd.draw(f, left_chunks[0])?;
         self.index.draw(f, left_chunks[1])?;
         self.diff.draw(f, chunks[1])?;
+        self.draw_branch_state(f, &left_chunks);
 
         Ok(())
     }
@@ -137,14 +145,55 @@ impl Status {
                 key_config.clone(),
                 false,
             ),
-            git_diff: AsyncDiff::new(sender.clone()),
+            git_diff: AsyncDiff::new(sender),
             git_status_workdir: AsyncStatus::new(sender.clone()),
             git_status_stage: AsyncStatus::new(sender.clone()),
             git_action_executed: false,
+            git_branch_state: None,
+            git_branch_name: cached::BranchName::new(CWD),
             key_config,
         }
     }
 
+    fn draw_branch_state<B: tui::backend::Backend>(
+        &self,
+        f: &mut tui::Frame<B>,
+        chunks: &[tui::layout::Rect],
+    ) {
+        if let Some(branch_name) = self.git_branch_name.last() {
+            let ahead_behind =
+                if let Some(state) = &self.git_branch_state {
+                    format!(
+                        "\u{2191}{} \u{2193}{} ",
+                        state.ahead, state.behind,
+                    )
+                } else {
+                    String::new()
+                };
+            let w = Paragraph::new(format!(
+                "{}{{{}}}",
+                ahead_behind, branch_name
+            ))
+            .alignment(Alignment::Right);
+
+            let mut rect = if self.index_wd.focused() {
+                let mut rect = chunks[0];
+                rect.y += rect.height.saturating_sub(1);
+                rect
+            } else {
+                chunks[1]
+            };
+
+            rect.x += 1;
+            rect.width = rect.width.saturating_sub(2);
+            rect.height = rect
+                .height
+                .saturating_sub(rect.height.saturating_sub(1));
+
+            f.render_widget(w, rect);
+        }
+    }
+
     fn can_focus_diff(&self) -> bool {
         match self.focus {
             Focus::WorkDir => self.index_wd.is_file_seleted(),
@@ -206,6 +255,8 @@ impl Status {
 
     ///
     pub fn update(&mut self) -> Result<()> {
+        self.git_branch_name.lookup().map(Some).unwrap_or(None);
+
         if self.is_visible() {
             self.git_diff.refresh()?;
             self.git_status_workdir.fetch(StatusParams::new(
@@ -215,7 +266,7 @@ impl Status {
             self.git_status_stage
                 .fetch(StatusParams::new(StatusType::Stage, true))?;
 
-            self.index_wd.update()?;
+            self.check_branch_state();
         }
 
         Ok(())
@@ -323,6 +374,51 @@ impl Status {
             true
         }
     }
+
+    fn push(&self) {
+        if let Some(branch) = self.git_branch_name.last() {
+            self.queue
+                .borrow_mut()
+                .push_back(InternalEvent::Push(branch));
+        }
+    }
+
+    fn fetch(&self) {
+        if let Some(branch) = self.git_branch_name.last() {
+            match sync::fetch_origin(CWD, branch.as_str()) {
+                Err(e) => {
+                    self.queue.borrow_mut().push_back(
+                        InternalEvent::ShowErrorMsg(format!(
+                            "fetch error:\n{}",
+                            e
+                        )),
+                    );
+                }
+                Ok(bytes) => {
+                    self.queue.borrow_mut().push_back(
+                        InternalEvent::ShowErrorMsg(format!(
+                            "fetched:\n{} B",
+                            bytes
+                        )),
+                    );
+                }
+            }
+        }
+    }
+
+    fn check_branch_state(&mut self) {
+        self.git_branch_state =
+            self.git_branch_name.last().and_then(|branch| {
+                sync::branch_compare_upstream(CWD, branch.as_str())
+                    .ok()
+            });
+    }
+
+    fn can_push(&self) -> bool {
+        self.git_branch_state
+            .as_ref()
+            .map_or(true, |state| state.ahead > 0)
+    }
 }
 
 impl Component for Status {
@@ -337,6 +433,20 @@ impl Component for Status {
                 force_all,
                 self.components().as_slice(),
             );
+
+            out.push(CommandInfo::new(
+                strings::commands::open_branch_select_popup(
+                    &self.key_config,
+                ),
+                true,
+                true,
+            ));
+
+            out.push(CommandInfo::new(
+                strings::commands::status_push(&self.key_config),
+                self.can_push(),
+                true,
+            ));
         }
 
         {
@@ -362,14 +472,6 @@ impl Component for Status {
             ));
         }
 
-        out.push(CommandInfo::new(
-            strings::commands::open_branch_create_popup(
-                &self.key_config,
-            ),
-            true,
-            true,
-        ));
-
         out.push(
             CommandInfo::new(
                 strings::commands::select_status(&self.key_config),
@@ -446,10 +548,16 @@ impl Component for Status {
                     && !self.index_wd.is_empty()
                 {
                     self.switch_focus(Focus::WorkDir)
-                } else if k == self.key_config.create_branch {
+                } else if k == self.key_config.select_branch {
                     self.queue
                         .borrow_mut()
-                        .push_back(InternalEvent::CreateBranch);
+                        .push_back(InternalEvent::SelectBranch);
+                    Ok(true)
+                } else if k == self.key_config.push {
+                    self.push();
+                    Ok(true)
+                } else if k == self.key_config.fetch {
+                    self.fetch();
                     Ok(true)
                 } else {
                     Ok(false)
